/*******************************************************************************
 * Copyright (c) 2011 itemis AG (http://www.itemis.eu) and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.eclipse.xtext.example.domainmodel.validation;

import static com.google.common.collect.Lists.*;

import java.util.List;

import org.eclipse.emf.ecore.EPackage;
import org.eclipse.xtext.common.types.util.TypeReferences;
import org.eclipse.xtext.example.domainmodel.domainmodel.DomainmodelPackage;
import org.eclipse.xtext.example.domainmodel.domainmodel.Entity;
import org.eclipse.xtext.example.domainmodel.domainmodel.Feature;
import org.eclipse.xtext.example.domainmodel.domainmodel.PackageDeclaration;
import org.eclipse.xtext.naming.IQualifiedNameProvider;
import org.eclipse.xtext.util.Strings;
import org.eclipse.xtext.validation.Check;
import org.eclipse.xtext.validation.ValidationMessageAcceptor;
import org.eclipse.xtext.xbase.XbasePackage;
import org.eclipse.xtext.xbase.validation.XbaseJavaValidator;

import com.google.inject.Inject;

public class DomainmodelJavaValidator extends XbaseJavaValidator {

	@Inject
	private TypeReferences references;

	@Inject 
	IQualifiedNameProvider qualifiedNameProvider;

	@Check
	public void factoryClassPresent(Entity entity) {
		String name = qualifiedNameProvider.getFullyQualifiedName(entity) + "Factory";
		if (references.findDeclaredType(name, entity) == null) {
            warning("Missing factory class for entity '" + entity.getName() + "': " + name , 
            		DomainmodelPackage.Literals.ENTITY__NAME,
            		ValidationMessageAcceptor.INSIGNIFICANT_INDEX,
            		IssueCodes.MISSING_FACTORY_CLASS, 
            		entity.getName());
		}
	}
	
	@Check
	public void interfaceClassPresent(Entity entity) {
		String name = qualifiedNameProvider.getFullyQualifiedName(entity) + "Interface";
		if (references.findDeclaredType(name, entity) == null) {
			warning("Missing interface class for entity '" + entity.getName() + "': " + name , 
					DomainmodelPackage.Literals.ENTITY__NAME,
					ValidationMessageAcceptor.INSIGNIFICANT_INDEX,
					IssueCodes.MISSING_FACTORY_CLASS, 
					entity.getName());
		}
	}
	
    @Check
    public void checkTypeNameStartsWithCapital(Entity entity) {
        if (!Character.isUpperCase(entity.getName().charAt(0))) {
            warning("Name should start with a capital", 
            		DomainmodelPackage.Literals.ENTITY__NAME,
            		ValidationMessageAcceptor.INSIGNIFICANT_INDEX,
            		IssueCodes.INVALID_TYPE_NAME, 
            		entity.getName());
        }
    }

    @Check
    public void checkFeatureNameStartsWithLowercase(Feature feature) {
        if (!Character.isLowerCase(feature.getName().charAt(0))) {
            warning("Name should start with a lowercase", 
            		DomainmodelPackage.Literals.FEATURE__NAME,
            		ValidationMessageAcceptor.INSIGNIFICANT_INDEX,
            		IssueCodes.INVALID_FEATURE_NAME, 
            		feature.getName());
        }
    }
    
    @Check
    public void checkPackage(PackageDeclaration packages) {
    	if(Strings.isEmpty(packages.getName())) {
    		error("Name cannot be empty", DomainmodelPackage.Literals.PACKAGE_DECLARATION__NAME);
    	}
        if (packages.getName().equals("java")) {
            error("Invalid package name", 
            		DomainmodelPackage.Literals.PACKAGE_DECLARATION__NAME);
        }
    }
    
    @Override
    protected boolean supportsCheckedExceptions() {
    	return false;
    }
    
	@Override
	protected List<EPackage> getEPackages() {
		return newArrayList(DomainmodelPackage.eINSTANCE, XbasePackage.eINSTANCE);
	}
}
